local vt      = require "lib.vt2000"

events = {}
local lastMouseMovedTime = 0
local mouseMoveDelay = 0.1 -- Set the delay in seconds

function events.add( udp, ip, port, bindings)
	 local actions = {}

	 for key, actionData in pairs(bindings) do
			local eventType = actionData.eventType or "press"  -- Default to "press" if not specified
			local isMouseBinding = key:match("mouse") ~= nil
			local isKeyBinding = key:match("mouse") == nil

			if isKeyBinding then
				 local modifiers = {}
				 local keyName = key:match("[^+]+$")
				 local ctrl = key:match("ctrl") ~= nil
				 local shift = key:match("shift") ~= nil

				 if ctrl then table.insert(modifiers, "ctrl") end
				 if shift then table.insert(modifiers, "shift") end

				 actions[key] = {
						type = "key",
						modifiers = modifiers,
						key = keyName,
						action = actionData.action,
						eventType = eventType
				 }
			elseif isMouseBinding then
				 local button = key:match("%d+")
				 actions[key] = {
						type = "mouse",
						button = tonumber(button),
						action = actionData.action,
						eventType = eventType
				 }
			end
	 end

	 love.keypressed = function(pressedKey, scancode)
			for _, data in pairs(actions) do
				 if data.type == "key" and data.eventType == "press" then
						local match = true
						match = match and (love.keyboard.isDown("lctrl", "rctrl") == table.contains(data.modifiers, "ctrl"))
						match = match and (love.keyboard.isDown("lshift", "rshift") == table.contains(data.modifiers, "shift"))
						match = match and (pressedKey == data.key)

						if match then
							 executeAction(data.action)
						end
				 else
						udp:sendto("keyreleased: " .. pressedKey .. " " .. scancode , ip, port)
				 end
			end
	 end

	 love.keyreleased = function(releasedKey, scancode)
			for _, data in pairs(actions) do
				 if data.type == "key" and data.eventType == "release" then
						local match = true
						match = match and (releasedKey == data.key)

						if match then
							 executeAction(data.action)
						end
				 else
						udp:sendto("keyreleased: " .. releasedKey .. " " .. scancode , ip, port)
				 end
			end
	 end

	 love.mousepressed = function(x, y, button)
			for _, data in pairs(actions) do
				 if data.type == "mouse" and data.eventType == "press" and data.button == button then
						executeAction(data.action)
				 else
						local row, col = vt.getCoords(font, x, y)
						udp:sendto("mousepressed: " .. row .. "x" .. col, ip, port)
				 end
			end
	 end

	 love.mousereleased = function(x, y, button)
			for _, data in pairs(actions) do
				 if data.type == "mouse" and data.eventType == "release" and data.button == button then
						executeAction(data.action)
				 else
						local row, col = vt.getCoords(font, x, y)
						udp:sendto("mousereleased: " .. row .. "x" .. col, ip, port)
				 end
			end
	 end

	 -- love.mousemoved = function(x, y, dx, dy, istouch)
	 -- 		row, col = vt.getCoords(font, x, y)
	 -- 		udp:sendto("mousemoved: " .. row .. "x" .. col, ip, port)
	 -- end

	 -- mousemoved with a timer
	 love.mousemoved = function(x, y)
			local currentTime = love.timer.getTime()
			if currentTime - lastMouseMovedTime >= mouseMoveDelay then
				 local row, col = vt.getCoords(font, x, y)
				 udp:sendto("mousemoved: " .. row .. "x" .. col, ip, port)
				 lastMouseMovedTime = currentTime
			end
	 end

	 love.textinput = function(text)
			udp:sendto("textinput: " .. text, ip, port)
	 end

	 love.resize = function()
			-- Update the grid based on the new window size
			rows, cols = vt.updateGrid(font)

			-- Resize the character table
			vt.resizeCharacterTable(charTable, rows, cols)

			-- Populate the character table with random characters for demonstration
			--populateCharacterTable(rows, cols)

			udp:sendto("resize: " .. rows .. "x".. cols, ip, port)
	 end

	 function table.contains(table, element)
			for _, value in pairs(table) do
				 if value == element then
						return true
				 end
			end
			return false
	 end

	 function executeAction(action)
			local func = action
			if type(func) == "function" then
				 func()
			else
				 executeFunctionFromString(func .. "()")
			end
	 end
end

function executeFunctionFromString(funcString)
	 local func, err = loadstring(funcString)
	 if not func then
			print("Error loading function:", err)
			return
	 end

	 setfenv(func, _G)  -- Set the environment to the global scope
	 local success, result = pcall(func)
	 if not success then
			print("Error calling function:", result)
	 end
end

return events
