--local h       = require "lib.helper"

local vt2000 = {}

function vt2000.updateGrid(font)
	 -- Calculate the number of rows and columns based on the window size and font metrics
	 local textWidth = font:getWidth("A") -- Width of the character 'A'
	 local textHeight = font:getHeight()

	 local cols = math.floor(love.graphics.getWidth() / textWidth)
	 local rows = math.floor(love.graphics.getHeight() / textHeight)

	 return rows, cols
end

function vt2000.getCoords(font, x, y)
		local textWidth = font:getWidth("A") -- Width of the character 'A'
		local textHeight = font:getHeight()

    local col = math.ceil(x / textWidth)
    local row = math.ceil(y / textHeight)

    return row, col
end

function vt2000.getCurrentRows(charTable)
	 if charTable == nil then
			return 0
	 else
			return #charTable
	 end
end

function vt2000.getCurrentCols(charTable)
	 local currentRows = vt2000.getCurrentRows(charTable)
	 return currentRows > 0 and #charTable[currentRows] or 0
end

function vt2000.getColor(charTable, row, col)
	 if charTable[row] then
			local data = charTable[row][col]
			if data then
				 return data[2]
			end
	 end
	 -- Return a default color if the specified field doesn't exist
	 return {255, 255, 255, 255}
end

function vt2000.getBg(charTable, row, col)
	 if charTable[row] then
			local data = charTable[row][col]
			if data then
				 return data[3]
			end
	 end
	 -- Return a default color if the specified field doesn't exist
	 return {0, 0, 0, 255}
end

function vt2000.getChar(charTable, row, col)
	 if charTable[row] then
			local data = charTable[row][col]
			if data then
				 return data[1]
			end
	 end
	 -- Return a default char if the specified field doesn't exist
	 return " "
end

function vt2000.setRow(charTable, row, char, color, bg)
	 if charTable[row] then
			local maxCols = vt2000.getCurrentCols(charTable)

			for i = 1, maxCols do
				 charTable[row][i] = { char, color, bg }
			end
	 end
end

function vt2000.setCol(charTable, col, char, color, bg)
	 local maxRows = vt2000.getCurrentRows(charTable)

	 for i = 1, maxRows do
			charTable[i][col] = { char, color, bg}
	 end
end

function vt2000.setRowRange(charTable, row, startCol, endCol, char, color, bg)
    local maxCols = vt2000.getCurrentCols(charTable)

    -- Ensure the range is within the valid bounds
    startCol = math.max(1, startCol)
    endCol = math.min(maxCols, endCol)

    for j = startCol, endCol do
        charTable[row][j] = { char, color, bg }
    end
end

function vt2000.setColumnRange(charTable, col, startRow, endRow, char, color, bg)
    local maxRows = vt2000.getCurrentRows(charTable)

    -- Ensure the range is within the valid bounds
    startRow = math.max(1, startRow)
    endRow = math.min(maxRows, endRow)

    for i = startRow, endRow do
        charTable[i][col] = { char, color, bg }
    end
end

function vt2000.insertCharacter(charTable, row, col, char, color, bg)
	 if charTable[row] and col > 0 and col <= vt2000.getCurrentCols(charTable) then
			charTable[row][col] = { string.sub(char, 1, 1), color, bg }
	 end
end

function vt2000.insertString(charTable, row, col, str, color, bg)
	 if charTable[row] and col > 0 then
			local strLen = #str
			local maxCols = vt2000.getCurrentCols(charTable)

			for i = 1, strLen do
				 if col + i - 1 <= maxCols then
						charTable[row][col + i - 1] = { str:sub(i, i), color, bg}
				 else
						break  -- Stop inserting if we reach the end of the row
				 end
			end
	 end
end

-- clears the whole screen
function vt2000.clear(charTable)
	 for i = 1, #charTable do
			for j = 1, vt2000.getCurrentCols(charTable) do
				 charTable[i][j] = {} -- empty table
			end
	 end
end

function vt2000.shiftLinesDown(charTable)
    local maxRows = vt2000.getCurrentRows(charTable)
    local maxCols = vt2000.getCurrentCols(charTable)

    -- Iterate from the bottom to the top, copying each row from the row above
    for row = maxRows - 1, 1, -1 do
        for col = 1, maxCols do
            charTable[row + 1][col] = { charTable[row][col][1], charTable[row][col][2], charTable[row][col][3] }
        end
    end

    -- Clear the first row
    for col = 1, maxCols do
        charTable[1][col] = { " ", nil, nil }
    end
end

function vt2000.resizeCharacterTable(charTable, rows, cols)
	 local currentRows = #charTable
	 local currentCols = currentRows > 0 and #charTable[1] or 0

	 -- Adjust the number of rows based on window size
	 if rows < currentRows then
			for i = currentRows, rows + 1, -1 do
				 table.remove(charTable, i)
			end
	 elseif rows > currentRows then
			for i = currentRows + 1, rows do
				 charTable[i] = {}
				 for j = 1, currentCols do
						charTable[i][j] = charTable[i][j] or " "
				 end
			end
	 end

	 -- Adjust the number of columns based on window size
	 if cols < currentCols then
			for i = 1, rows do
				 for j = currentCols, cols + 1, -1 do
						table.remove(charTable[i], j)
				 end
			end
	 elseif cols > currentCols then
			for i = 1, rows do
				 for j = currentCols + 1, cols do
						if j ~= cols then
							 charTable[i][j] = charTable[i][j] or " "
						end
				 end
			end
	 end
end

function vt2000.drawGrid(charTable, font, cols, rows)
	 local textWidth = font:getWidth("A")
	 local textHeight = font:getHeight()

	 for i = 1, rows do
			for j = 1, cols do
				 local data = charTable[i][j] or { " ", {255, 255, 255, 255} }

				 local char = data[1] or ""
				 local color = data[2] or {255, 255, 255, 255} -- Use default color if nil
				 local bg = data[3]

				 local x = (j - 1) * textWidth
				 local y = (i - 1) * textHeight

				 -- bg
				 if bg ~= nil then
						love.graphics.setColor(love.math.colorFromBytes(bg))
						love.graphics.rectangle("fill", x, y, textWidth, textHeight)
						love.graphics.setColor(love.math.colorFromBytes(255, 255, 255, 255)) -- Reset color to default
				 end

				 -- drawgrid
				 --love.graphics.rectangle("line", x, y, textWidth, textHeight)

				 love.graphics.setColor(love.math.colorFromBytes(color))
				 love.graphics.print(char, x, y)
				 love.graphics.setColor(love.math.colorFromBytes(255, 255, 255, 255)) -- Reset color to default
			end
	 end
end

return vt2000
