-- keymap.lua

local keymap = {}

function keymap.add(bindings)
  local actions = {}

  for key, actionData in pairs(bindings) do
    local eventType = actionData.eventType or "press"  -- Default to "press" if not specified
    local isMouseBinding = key:match("mouse") ~= nil
    local isKeyBinding = key:match("mouse") == nil

    if isKeyBinding then
      local modifiers = {}
      local keyName = key:match("[^+]+$")
      local ctrl = key:match("ctrl") ~= nil
      local shift = key:match("shift") ~= nil

      if ctrl then table.insert(modifiers, "ctrl") end
      if shift then table.insert(modifiers, "shift") end

      actions[key] = {
        type = "key",
        modifiers = modifiers,
        key = keyName,
        action = actionData.action,
        eventType = eventType
      }
    elseif isMouseBinding then
      local button = key:match("%d+")
      actions[key] = {
        type = "mouse",
        button = tonumber(button),
        action = actionData.action,
        eventType = eventType
      }
    end
  end

  love.keypressed = function(pressedKey, scancode, isrepeat)
    for _, data in pairs(actions) do
      if data.type == "key" and data.eventType == "press" then
        local match = true
        match = match and (love.keyboard.isDown("lctrl", "rctrl") == table.contains(data.modifiers, "ctrl"))
        match = match and (love.keyboard.isDown("lshift", "rshift") == table.contains(data.modifiers, "shift"))
        match = match and (pressedKey == data.key)

        if match then
          executeAction(data.action)
        end
      end
    end
  end

  love.keyreleased = function(releasedKey, scancode)
    for _, data in pairs(actions) do
      if data.type == "key" and data.eventType == "release" then
        local match = true
        match = match and (releasedKey == data.key)

        if match then
          executeAction(data.action)
        end
      end
    end
  end

  love.mousepressed = function(x, y, button, istouch, presses)
    for _, data in pairs(actions) do
      if data.type == "mouse" and data.eventType == "press" and data.button == button then
        executeAction(data.action)
      end
    end
  end

  love.mousereleased = function(x, y, button, istouch, presses)
    for _, data in pairs(actions) do
      if data.type == "mouse" and data.eventType == "release" and data.button == button then
        executeAction(data.action)
      end
    end
  end

  function table.contains(table, element)
    for _, value in pairs(table) do
      if value == element then
        return true
      end
    end
    return false
  end

  function executeAction(action)
    local func = action
    if type(func) == "function" then
      func()
    else
      executeFunctionFromString(func .. "()")
    end
  end
end

function executeFunctionFromString(funcString)
  local func, err = loadstring(funcString)
  if not func then
    print("Error loading function:", err)
    return
  end

  setfenv(func, _G)  -- Set the environment to the global scope
  local success, result = pcall(func)
  if not success then
    print("Error calling function:", result)
  end
end

return keymap
